module Main where

import AFrob
import AFrobRobotSim

------------------------------------------------------------------------------
-- Controllers for team 1 here
------------------------------------------------------------------------------

player1_1 :: SimbotController
player1_1 = moreira 1.0  --ok  


player1_2 :: SimbotController
player1_2 = fernandoAguiar 1.5


player1_3 :: SimbotController
player1_3 = figo 2.0


------------------------------------------------------------------------------
-- Controllers for team 2 here
------------------------------------------------------------------------------

player2_1 :: SimbotController
player2_1 = baia 1.0 


player2_2 :: SimbotController
player2_2 =  fernandoAguiar 1.5


player2_3 :: SimbotController
player2_3 = figo 2.0


------------------------------------------------------------------------------
-- Some potentially useful controllers

------------------------------------------------------------------------------

-- Goes towards the ball.
rcGoToBall :: Velocity -> SimbotController
rcGoToBall vd rps = proc sbi -> do
    let (phi, d) = head (aotBalls sbi ++ [(0.0,0.0)])
        h        = odometryHeading sbi
    rcHeadingAux rps -< (sbi, vd, h + phi)

rcGoToBall2 :: Velocity -> SimbotController
rcGoToBall2 vd rps =
    let loop = switch (rcGoToBall vd rps &&& rsStuck) $ \_ ->
               switch (constant (mrFinalize (ddVelTR (-vd) 0.3))
                       &&& after 2.5 ()) $ \_ ->
               loop
    in
        loop


-- Goes in the desired heading at the desired speed.
rcHeadingAux :: SimbotProperties -> 
                SF (SimbotInput,Velocity,Heading) SimbotOutput
rcHeadingAux rps = 
    let vMax = rpWSMax rps
        k    = 2
    in proc (sbi,vel,hd) -> do
         let vel' = lim vMax vel
         let phi = normalizeAngle (hd - odometryHeading sbi)
         let vel'' = (1 - abs phi / pi) * vel'
         returnA -< mrFinalize (ddVelTR vel'' (k*phi))

lim m y = max (-m) (min m y)


-- Stops the robot.
rcStop :: SimbotController
rcStop _ = constant (mrFinalize ddBrake)



-------------------------
-----Meu-----------------
-------------------------

fernandoAguiar :: Velocity -> SimbotController
fernandoAguiar v rps = ate_bater `switch` const (fernandoAguiar (-v) rps)
		     where
                      ate_bater = proc sbi -> do
					bateu <- rsStuck -< sbi
                                        returnA -< (mrFinalize (ddVelDiff v v),bateu)


moreira :: Velocity -> SimbotController
moreira v  rps = coloca `switch` vira 
                       	where
                        coloca = proc sbi -> do
                                      let posicao = odometryHeading sbi
				          inicio = (posicao == inic1)
				      sinal <- after 0.2  () -< () 
                                      let para = sinal `tag`  inicio
                        	      returnA -< if inicio then  (mrFinalize ddBrake,para)  else (mrFinalize $ ddVelDiff 1 1,para) 
                        vira True   = (constant (mrFinalize (ddVelDiff (-v) 0))  &&& after 0.8 ()) `switch` \_ -> fernandoAguiar (-2*v) rps
                        vira False  = fernandoAguiar v rps



baia :: Velocity -> SimbotController
baia v  rps = coloca `switch` vira 
                       	where
                        coloca = proc sbi -> do
                                      let posicao = odometryHeading sbi
				          inicio = (posicao == inic2)
				      sinal <- after 0.2  () -< () 
                                      let para = sinal `tag`  inicio
                        	      returnA -< if inicio then  (mrFinalize ddBrake,para)  else (mrFinalize $ ddVelDiff 1 1,para) 
                        vira True   = (constant (mrFinalize (ddVelDiff (-v) 0))  &&& after 0.8 ()) `switch` \_ -> fernandoAguiar (-2*v) rps
                        vira False  = fernandoAguiar v rps





figo :: Velocity -> SimbotController
figo v rps = switch  (rcFinta (-v) rps &&& after 1 ()) 
             (\_->  switch (fernandoAguiar v rps &&& after 2.0 ())  
             (\_-> switch  (rcGoToBall2 v rps &&& after 10.0 () ) (\_-> figo (-v) rps) )) 
             



rcFinta :: Velocity -> SimbotController
rcFinta v rps = let vMax = rpWSMax rps
		    rMax = 2 * (vMax - v) / rpDiameter rps
	        in constant (mrFinalize $ ddVelTR v rMax) 


inic1,inic2 :: Heading
inic1 = 0
inic2 = pi
------------------------------------------------------------------------------
-- Main
------------------------------------------------------------------------------

main = playSoccer [player1_1, player1_2, player1_3]
                  [player2_1, player2_2, player2_3]
